﻿using System;
using System.Text;
using System.Windows.Forms;
using System.Web.ClientServices;
using System.Web.ClientServices.Providers;
using System.Web.Security;


namespace ClientApplication
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);
        }

        // save settings when the form is about to close
        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveSettings();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Validate the user
            if (!ValidateUser()) 
                return;

            // Fill user information
            DisplayUserInfo();

            // File user roles
            DisplayUserRoles();

            // Bind the Web settings to form controls
            BindWebSettings();
        }

        private void BindWebSettings()
        {
            try
            {
                // Add binding to the last login date
                this.lblLastLoginDate.DataBindings.Add("Text", Properties.Settings.Default, "LastLoginDate");

                // Set the background color to the one stored in the application settings
                if (string.IsNullOrEmpty((Properties.Settings.Default.Back_Color)))
                    Properties.Settings.Default.Back_Color = "#909090";

                this.BackColor = System.Drawing.ColorTranslator.FromHtml((Properties.Settings.Default.Back_Color));
            }
            catch (System.Net.WebException)
            {
                MessageBox.Show("Unable to access the Web Settings service.",
                    "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        /// <summary>
        /// Displays user roles in a grid
        /// </summary>
        private void DisplayUserRoles()
        {
            // Get the ClientRolePrincipal
            ClientRolePrincipal principal =
                (ClientRolePrincipal)System.Threading.Thread.CurrentPrincipal;

            // Get an instance of the ClientRoleProvider
            ClientRoleProvider roleProvider = (ClientRoleProvider)Roles.Provider;

            // Get user roles
            string[] roles = roleProvider.GetRolesForUser(principal.Identity.Name);

            panel1.Visible = roles.Length >= 1;
            if (panel1.Visible)
            {
                foreach (string s in roles)
                    this.dataGridView1.Rows.Add(s);
            }
        }

        /// <summary>
        /// Binds user information
        /// to a label on the form
        /// </summary>
        private void DisplayUserInfo()
        {
            // Get the ClientFormsIdentity
            ClientFormsIdentity user =
                (ClientFormsIdentity)System.Threading.Thread.CurrentPrincipal.Identity;

            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(
                "Welcome {0} !! \r\n\r\n" +
                "Authentication Type: {1} \r\n", user.Name, user.AuthenticationType);
            this.label1.Text = sb.ToString();
        }

        /// <summary>
        /// Validates a user by using the Login form which 
        /// implements the IClientFormsAuthenticationCredentialsProvider 
        /// interface.
        /// </summary>
        /// <returns></returns>
        private bool ValidateUser()
        {
            bool isAuthenticated = false;
            try
            {
                isAuthenticated = Membership.ValidateUser(string.Empty, string.Empty);

                try
                {
                    // Save last login date
                    Properties.Settings.Default.LastLoginDate = DateTime.Now;
                }
                catch (System.Net.WebException)
                {
                    MessageBox.Show("Unable to access the Web Settings service.",
                                    "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (System.Net.WebException ex)
            {
                // a WebException is raised when the authentication service
                // cannot be reached
                if (DialogResult.OK == MessageBox.Show(
                    "Unable to access the authentication service", "Not logged in",
                    MessageBoxButtons.OK, MessageBoxIcon.Error))
                Application.Restart();
            }

            if (!isAuthenticated)
            {
                MessageBox.Show("Unable to authenticate credentials", "Not logged in",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Exit();
            }

            return isAuthenticated;
        }

        /// <summary>
        /// Stores Web Settings into the Profile Service
        /// </summary>
        private void SaveSettings()
        {
            // Check if the user is logged in to the Forms Authentication
            // service, i.e. Authentication type is "ClientForms", if not
            // return without saving any settings.
            if (!System.Threading.Thread.CurrentPrincipal.Identity.AuthenticationType.Equals("ClientForms"))
                return;

            try
            {
                Properties.Settings.Default.Save();
            }
            catch (System.Net.WebException ex)
            {
                // This means you are logged out
                if (ex.Message.Contains("You must log on to call this method."))
                {
                    MessageBox.Show(
                        "Your session has expired. Please login again to be able to save" +
                        "your settings.", "Saving Web Settings");

                    try
                    {
                        // Show the Login form for the user to enter his/her credentials
                        // to login again 
                        if (!Membership.ValidateUser(String.Empty, String.Empty))
                        {
                            MessageBox.Show("Unable to authenticate. " +
                                "Settings were not saved on the remote service.",
                                "Not logged in", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                        }
                        else
                        {
                            // Try again saving the settings
                            // after the user has been authenticated
                            SaveSettings();
                        }
                    }
                    catch (System.Net.WebException)
                    {
                        MessageBox.Show(
                            "Unable to access the authentication service. " +
                            "Settings were not saved on the remote service.",
                            "Not logged in", MessageBoxButtons.OK,
                            MessageBoxIcon.Warning);
                    }
                }
                else
                {
                    MessageBox.Show(
                        "Unable to access the Web settings service. " +
                        "Settings were not saved on the remote service.",
                        "Not logged in", MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                }
            }
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            // Save the Web settings first
            SaveSettings();
            
            // Get an instance of the ClientFormsAuthenticationMembershipProvider
            // to use the Logout function which is not present on Membership.Provider
            ClientFormsAuthenticationMembershipProvider clientFormAuthProvider =
                (ClientFormsAuthenticationMembershipProvider)Membership.Provider;

            try
            {
                clientFormAuthProvider.Logout();
            }
            catch (System.Net.WebException ex)
            {
                MessageBox.Show("Unable to access the authentication service." +
                    Environment.NewLine + "Logging off locally only.",
                    "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                
                ConnectivityStatus.IsOffline = true;
                clientFormAuthProvider.Logout();
                ConnectivityStatus.IsOffline = false;
            }

            Application.Restart();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            // Set the IsOffline property on the ConnectivityStatus
            ConnectivityStatus.IsOffline = this.chkWorkOffline.Checked;
            
            // If the application is moving from offline to 
            // online mode, revalidate again using the credentials
            // stored locally
            if (!ConnectivityStatus.IsOffline)
            {
                try
                {
                    // Revalidate the user silently
                    ClientFormsIdentity user = (ClientFormsIdentity)System.Threading.Thread.CurrentPrincipal.Identity;
                    user.RevalidateUser();

                    // If the user have made any changes when the application
                    // was offline, save them to the Web Settings on the remote 
                    // profile service.
                    SaveSettings();

                    // If any changes occured to the profile settings on the
                    // remote service, reload them to the application.
                    Properties.Settings.Default.Reload();
                }
                catch (System.Net.WebException)
                {
                    MessageBox.Show(
                        "Unable to access the authentication service. " +
                        Environment.NewLine + "Application will be kept in offline mode.",
                        "Warning", MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    this.chkWorkOffline.Checked = true;
                }
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            // Save the changes
            Properties.Settings.Default.Back_Color = this.txtBackColor.Text;

            // Refresh the background of the form
            this.BackColor = System.Drawing.ColorTranslator.FromHtml((Properties.Settings.Default.Back_Color));

            MessageBox.Show("Web Settings changed succeffully.");
        }

    }
}